const express = require('express');

const pubApp = express();

const libs = `${process.cwd()}/libs`;

const config = require(`${libs}/config`);

const log = require(`${libs}/log`)(module);

const Publication = require(`${libs}/models/publication`);

const File = require(`${libs}/models/file`);

const response = require(`${libs}/middlewares/response`);

const email = require(`${libs}/middlewares/email`);

const passport = require('passport');

const fileWorker = require('./file.controller.js');

let upload = require('../middlewares/multer.config.js');

const authorized = require(`${libs}/middlewares/authorize.js`);

function emailSyntax(email) {
  const regex = /^(([^<>()\[\]\.,;:\s@\"]+(\.[^<>()\[\]\.,;:\s@\"]+)*)|(\".+\"))@(([^<>()[\]\.,;:\s@\"]+\.)+[^<>()[\]\.,;:\s@\"]{2,})$/i;
  return regex.test(email);
}

pubApp.get('/', async (req, res, next) => {
  const page = parseInt(req.query.page) || 1; // Current page number
  const pageSize = parseInt(req.query.pageSize) || 5; // Number of items per page
  try {
    const totalCount = await Publication.count();
    const offset = (page - 1) * pageSize;

    const publis = await Publication.findAll({
      offset,
      limit: pageSize,
    });

    res.json({
      page,
      pageSize,
      totalCount,
      data: publis,
    });
  } catch (error) {
    console.error(error);
    res.status(500).json({ error: 'An error occurred' });
  }
});

pubApp.get('/drafts', async (req, res, next) => {
  const page = parseInt(req.query.page) || 1; // Current page number
  const pageSize = parseInt(req.query.pageSize) || 5; // Number of items per page

  try {
    // Count total number of items
    const totalCount = await Publication.count({ where: {
      is_draft: true
    }});

    // Calculate offset based on page and pageSize
    const offset = (page - 1) * pageSize;

    // Query the database with pagination options
    const drafts = await Publication.findAll({
      offset,
      limit: pageSize,
      where: {
        is_draft: true
      }
    });

    res.json({
      page,
      pageSize,
      totalCount,
      data: drafts,
    });
  } catch (error) {
    console.error(error);
    res.status(500).json({ error: 'An error occurred' });
  }
});


pubApp.get('/:id', async (req, res, next) => {
  let pb = await Publication.findByPk(req.params.id).catch(function (err) {
    log.error(err);
    return next(err);}
  );
  if (!pb) {
      res.statusCode = 404;
      res.json({ msg: "A publicação não está cadastrada" });
    } 
  else {
    let publ = pb.toJSON();
    publ.Filename = null;
    const file_ = await File.findByPk(pb.upload).catch((err) => {
      log.error(err);
      return next(err);
    });
    if(file_){
      publ.Filename = file_.name;
      }
    req.result = publ;
    next();
    }
  }, response('publication'));

pubApp.post('/', passport.authenticate('bearer', { session: false }), authorized('criar publicacao'), upload.single('file'), async (req, res, next) => {
  let _file_id = null
  if(req.file){
    _file_id = await fileWorker.uploadFile(req.file);
    if(!_file_id)
      console.log("NAO ARQUIVO");}
  let data = JSON.parse(req.body.data);
  let pb = await Publication.create({
    id: 0,
    filter: data.categoria,
    title: data.title,
    authors: data.autores,
    organization: data.organizacao,
    year: data.ano,
    text: data.texto,
    link: data.link,
    upload: _file_id,
    is_draft: data.rascunho,
    is_homepage: data.homepage
  }).catch(function (err) {
    log.error(err);
    let errors = [];
    for (let errName in err.errors) {
      errors.push(err.errors[errName].message);
    }
    log.error(errors);
    res.statusCode = 400;
    return res.json({ err, errors });
    // handle error;
  });
  req.result = pb.toJSON();
  next();
}, response('publication'));

pubApp.post('/edit', passport.authenticate('bearer', { session: false }), authorized('editar publicacao'), upload.single('file'), async (req, res, next) => {
  let _file_id = null
  if(req.file){
    _file_id = await fileWorker.uploadFile(req.file);
    if(!_file_id)
      console.log("NAO ARQUIVO");}
  let data = JSON.parse(req.body.data);
  console.log(data);
  req.response = {'This is a test':'This is a test'};
  next();
}, response('publication'));

pubApp.put('/edit/:id', passport.authenticate('bearer', { session: false }), authorized('editar publicacao'), async (req, res, next) => {
  console.log(req);
  let pb = await Publication.findByPk(req.params.id).catch(function (err) {
    if (err) {
      log.error(err);
      return next({ err });
    }
  })
  if (!pb) {
    res.statusCode = 404;
    return next({
      err: {
        message: 'Publicação não encontrada'
      }
    });
  }
  pb.filter = req.body.categoria || pb.filter;
  pb.title = req.body.title || pb.title;
  pb.authors = req.body.autores || pb.authors;
  pb.organization= req.body.organizacao || pb.organization;
  pb.year= req.body.ano || pb.year;
  pb.text= req.body.texto || pb.text;
  pb.link= req.body.link || pb.link;
  pb.upload= req.body.upload || pb.upload;
  pb.is_homepage= req.body.homepage || pb.is_homepage;
  console.log(pb);
  pb.save().catch(err => {
    if (err) {
      log.error(err);
      return next({ message: 'Erro ao atualizar publicacao' });
    }
  })
  let p = pb.toJSON();
  res.json({ publication: p });

});

pubApp.delete('/:id', passport.authenticate('bearer', { session: false }), authorized('apagar publicacao'), async (req, res, next) => {
  await Publication.destroy({where:{id:req.params.id}}).catch(function (err) {
    if (err) {
        log.error(err);
        return next({ err });
    }
});
  return next({ msg: 'Publication Deleted', status: 200 });
});

module.exports = pubApp;
